#include "Execute.h"
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <QtGui/QMessageBox>

#define NOBODY 99

extern void Execute::execute (QWidget*     parent,
                              char* const  argv[])
{
   int  child = fork();
   if (child == -1) {
      // Still the parent because fork failed
      QString  text = QObject::tr("fork() failed; errno = %1 (%2)")
                         .arg(errno)
                         .arg(strerror(errno));
      QMessageBox::critical(/* parent  => */ parent,
                            /* caption => */ QObject::tr("Help"),
                            /* text    => */ text);
   } else if (child != 0) {
      // Still the parent: reap the child (which will exit promptly)
      int  status;
      waitpid(child, &status, 0);
   } else {
      // Now I'm the child
      int  grandchild = fork();
      if (grandchild == -1) {
         // Still the child because the second fork failed.  Bummer.
         fprintf(stderr, "second fork() failed; errno = %u (%s)\n",
                 errno, strerror(errno));
         _exit(1);
      } else if (grandchild != 0) {
         // Still the child: _exit() and leave the grandchild an orphan.
         _exit(0);
      } else {
         // Now I'm the grandchild
#undef SETUID_NOBODY
         // SETUID_NOBODY is irritating because it doesn't use the user's
         // web & mime settings.
#ifdef SETUID_NOBODY
         if (setgid(NOBODY) != 0) {
            fprintf(stderr, "setgid(NOBODY) failed; errno = %u (%s)\n",
                    errno, strerror(errno));
         }
         if (setuid(NOBODY) != 0) {
            fprintf(stderr, "setuid(NOBODY) failed; errno = %u (%s)\n",
                    errno, strerror(errno));
         }
#endif
         execv(argv[0], argv);
         fprintf(stderr, "execv(\"%s\", ...) failed; errno = %u (%s)\n", 
                 argv[0], errno, strerror(errno));
         _exit(1);
      }
   }
}
