#include "HScrollArea.h"
#include <QtGui/QScrollBar>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>
#include <QtGui/QX11Info>
#include <QtGui/QStyle>
#include <limits.h>

HScrollArea::HScrollArea(QWidget*  parent)
   : QScrollArea(parent),
     maxSize(INT_MAX, INT_MAX)
{
}

HScrollArea::HScrollArea(QSize     maxSize,
                         QWidget*  parent)
   : QScrollArea(parent),
     maxSize(maxSize)
{
}

HScrollArea::HScrollArea(int       maxColumns,
                         int       maxLines,
                         QWidget*  parent)
   : QScrollArea(parent),
     maxSize(QSize(maxColumns == -1 ? INT_MAX
                                    : maxColumns * fontMetrics().width('n'), 
                   maxLines   == -1 ? INT_MAX
                                    : maxLines   * fontMetrics().height()))
{
}

int HScrollArea::frameOnlyAroundContentsPixels() const
{
   if (style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents, 
                          0, this)) {
      return style()->pixelMetric(QStyle::PM_DefaultFrameWidth) * 2;
   } else {
      return 0;
   }
}

QSize  HScrollArea::sizeHint() const
{
   if (!widget()) return QScrollArea::sizeHint();

   QSize  widgetSize = widget()->sizeHint();

   int  w = (frameWidth()
             + widgetSize.width()
             + frameOnlyAroundContentsPixels()
             + verticalScrollBar()->sizeHint().width()
             + frameWidth());
   int  h = (frameWidth()
             + widgetSize.height()
             + frameOnlyAroundContentsPixels()
             + horizontalScrollBar()->sizeHint().height()
             + frameWidth());

   QRect  screen  = QApplication::desktop()->screenGeometry(x11Info().screen());
   return QSize(w, h).boundedTo(maxSize).boundedTo(QSize(screen.width()*9/10,
                                                         screen.height()*9/10));
}

void HScrollArea::forceResize()
{
   if (!widget()) return;

   widget()->resize(widget()->sizeHint());
   updateGeometry();
   widget()->resize(widget()->sizeHint());
}
