#ifndef HScrollArea_h
#define HScrollArea_h

#include <QtGui/QScrollArea>

// QTableView with these extensions:
//    Sets the preferred width & height to exactly enough to display the
//       interior widget, if possible, but reserving enough space that
//       gratuitous scrollbars won't be needed on one dimension if a
//       scrollbar for the other dimension ends up being necessary
//    Restricts the preferred width and height to a number of pixels
//       specified in the constructor (or -1 for no limit)
//    Fixes the sizeHint to include the extra pixels added if 
//       QStyle::SH_ScrollView_FrameOnlyAroundContents is enabled (as it is
//       for the Motif and CDE styles); the trolltech code neglected to
//       take that into account even when the scrollbar policies were
//       Qt::ScrollBarAlwaysOn
//    Add forceResize() function to force the Scroll Area to resize itself
//       based on a change in the sizeHint of its interior widget

class HScrollArea : public QScrollArea {
public:
   explicit HScrollArea(QWidget*  parent     = NULL);
   explicit HScrollArea(QSize     maxSize,
                        QWidget*  parent     = NULL);
   explicit HScrollArea(int       maxColumns,
                        int       maxLines,
                        QWidget*  parent     = NULL);
public:
   QSize sizeHint() const;
public:
   void forceResize();
protected:
   int frameOnlyAroundContentsPixels() const;
protected:
   QSize  maxSize;
};

#endif
