#include "HSortFilterProxyModel.h"
#include <Qt/qdatetime.h>

HSortFilterProxyModel::HSortFilterProxyModel(int       secondarySortColumn,
                                             QObject*  parent)
   : QSortFilterProxyModel(parent),
     secondarySortColumn(secondarySortColumn)
{
}

#define COMPARE(l,r,t)                                   \
   if (l.to##t() < r.to##t()) {                          \
      return true;                                       \
   } else if (l.to##t() == r.to##t()                     \
              && left.column() != secondarySortColumn) { \
      return lessThan(createSecondarySortIndex(left),    \
                      createSecondarySortIndex(right));  \
   } else {                                              \
      return false;                                      \
   }

bool HSortFilterProxyModel::lessThan(const QModelIndex&  left, 
                                     const QModelIndex&  right) const
{
   QVariant l = (left.model()  ? left.model()->data(left, Qt::DisplayRole)
                               : QVariant());
   QVariant r = (right.model() ? right.model()->data(right, Qt::DisplayRole)
                               : QVariant());

   switch (l.type()) {
   case QVariant::Int:       COMPARE(l, r, Int);
   case QVariant::UInt:      COMPARE(l, r, UInt);
   case QVariant::LongLong:  COMPARE(l, r, LongLong);
   case QVariant::ULongLong: COMPARE(l, r, ULongLong);
   case QVariant::Double:    COMPARE(l, r, Double);
   case QVariant::Char:      COMPARE(l, r, Char);
   case QVariant::Date:      COMPARE(l, r, Date);
   case QVariant::Time:      COMPARE(l, r, Time);
   case QVariant::DateTime:  COMPARE(l, r, DateTime);
   case QVariant::String:    COMPARE(l, r, String);
   default:                  COMPARE(l, r, String);
   }
}

QModelIndex  HSortFilterProxyModel::createSecondarySortIndex
   (const QModelIndex&  ix) const
{
   return ix.model()->index(ix.row(), secondarySortColumn);
}
