#ifndef HTableView_h
#define HTableView_h

#include <QtGui/QTableView>
#include <limits.h>

// QTableView with these extensions:
//    Sets the preferred width to exactly enough to display all columns
//    Sets the preferred height to the specified number of rows
//       or, if rows == -1, then the actual number of rows
//    Changes the decoration position to a value specified
//    Overrides sortByColumn with a version that defaults to ascending order
//    Overrides resizeColumnsToContents to resize the window containing the
//       table based on its new preferred size
//    Overrides sizeHintForColumn to use the maximum size of *all* the rows
//       in the column, and not just the visible ones
//    Added selectedRows() method which returns QModelIndexList with only
//       one index per row (useful when using selectionBehavior 
//       QAbstractItemView::SelectRows)

class HTableView : public QTableView {
public:
   explicit HTableView
      (QStyleOptionViewItem::Position  pos     = QStyleOptionViewItem::Left,
       int                             minRows = 0,
       int                             maxRows = INT_MAX,
       QWidget*                        parent  = NULL);
public:
   QModelIndexList  selectedRows() const;
public:
   QSize sizeHint() const;
   void  sortByColumn(int column);
public slots:
   void  resizeColumnsToContents();
protected:
   QStyleOptionViewItem  viewOptions() const;
   int sizeHintForColumn(int column) const;
protected:
   int frameOnlyAroundContentsPixels() const;
protected:
   QStyleOptionViewItem::Position  pos;
   mutable int                     minRows;
   mutable int                     maxRows;
};

#endif
