#include "HTextEdit.h"
#include <QtGui/QTextCursor>
#include <QtGui/QApplication>
#include <QtGui/QDesktopWidget>
#include <QtGui/QX11Info>
#include <QtGui/QFocusEvent>

HTextEdit::HTextEdit(QWidget*  parent)
   : QTextEdit(parent),
     columns(-1),
     lines(-1)
{
}

HTextEdit::HTextEdit(int       columns,
                     int       lines,
                     QWidget*  parent)
   : QTextEdit(parent),
     columns(columns),
     lines(lines)
{
}

QSize HTextEdit::sizeHint() const
{
   int  w = (columns == -1 ? QTextEdit::sizeHint().width()
                           : (frameWidth()
                              + fontMetrics().width('n') * columns
                              + frameWidth()));
   int  h = (lines == -1   ? QTextEdit::sizeHint().height()
                           : (frameWidth()
                              + (fontMetrics().lineSpacing() * lines)
                              + frameWidth()));

   QRect  screen  = QApplication::desktop()->screenGeometry(x11Info().screen());
   return QSize(w, h).boundedTo(QSize(screen.width()*9/10,
                                      screen.height()*9/10));
}

void HTextEdit::focusOutEvent(QFocusEvent*  ev)
{
   if (ev->reason() != Qt::PopupFocusReason
       && !(QApplication::activePopupWidget()
            && QApplication::activePopupWidget()->parentWidget() == this)) {
      emit editingFinished();
   }

   QTextEdit::focusOutEvent(ev);
}

void HTextEdit::appendCompact(const QString&  text)
{
   QTextEdit::append(text);

   QTextCursor cursor(document());
   cursor.beginEditBlock();
   cursor.movePosition(QTextCursor::End);
   QTextBlockFormat  tbf = cursor.blockFormat();
   tbf.setTopMargin(0);
   tbf.setBottomMargin(0);
   cursor.setBlockFormat(tbf);
   cursor.endEditBlock();
}
