#ifndef Package_h
#define Package_h

#include <Python.h>
#include <QtCore/QList>
#include <QtCore/QString>
#include <QtCore/QMetaType>

namespace Package {

   // The order of enumeration literals affects the sort order in the table.
   // See PackageTable::SortFilterProxyModel::lessThan.
   enum State {
      OUT_OF_DATE,
      NOT_INSTALLED,
      INSTALLED,     // implies up-to-date for primary (non-extra) Entries
   };         

   // The order of enumeration literals affects the sort order in the table.
   // See PackageTable::SortFilterProxyModel::lessThan.
   enum Action {
      UPDATE,
      INSTALL,
      REMOVE,
      NONE,
   };

#define ACTION_IMAGE(act) \
   ((act) == Package::UPDATE  ? QObject::tr("Update")  : \
    (act) == Package::INSTALL ? QObject::tr("Install") : \
    (act) == Package::REMOVE  ? QObject::tr("Remove")  : \
                                QObject::tr("ERROR"))

   // The order of enumeration literals affects the sort order in the table.
   // See PackageTable::SortFilterProxyModel::lessThan.
   enum Kind {
      SUITE,
      GROUP,
      PACKAGE,
   };

   typedef unsigned long long  Size;

   class Entry {
   public:
      Action     action;
      State      state;
      Kind       kind;
      QString    name;
      QString    verrel;
      QString    repo;
      QString    arch;
      Size       size;
      QString    sizeText;
      QString    summary;
      time_t     buildTime;
      QString    buildTimeText;
      PyObject*  obj;

      Entry*     next;

      // apply is used during YumIO::apply
      struct {
         Action  action; // action actually to be performed, including those
                         // from groups or conflict resolution
         Entry*  reason; // used for diagnosing action conflicts:
                         //    this:  action from direct request
                         //    group: action inherited from group request
                         //    NULL:  action from user conflict resolution
      } apply;

      explicit Entry() : size(0), obj(NULL), next(NULL) {}
      ~Entry() 
      { 
         Py_XDECREF(obj); 
         if (next != NULL) delete next;
      }
   };

   class EntryList : public QList<Entry*> {
   public:
      ~EntryList()
      {
         qDeleteAll(begin(), end());
      }
   };

};

Q_DECLARE_METATYPE(Package::Entry*)

#endif
