#include "PackageTableActionContextMenu.h"
#include "Share.h"

PackageTable::ActionContextMenu::ActionContextMenu(Model*  model)
   : QMenu(),
     noneIcon(),
     updateIcon(Share::absolute("icons/update.png")),
     installIcon(Share::absolute("icons/install.png")),
     removeIcon(Share::absolute("icons/remove.png")),
     model(model)
{
   updateAction  = new QAction(tr("&Update"), this);
   installAction = new QAction(tr("&Install"), this);
   removeAction  = new QAction(tr("&Remove"), this);
   noneAction    = new QAction(tr("&No Action"), this);

   updateAction->setIcon(updateIcon);
   installAction->setIcon(installIcon);
   removeAction->setIcon(removeIcon);
   noneAction->setIcon(noneIcon);

   addAction(updateAction);
   addAction(installAction);
   addAction(removeAction);
   addAction(noneAction);

   connect(updateAction, SIGNAL(triggered()), 
           this, SLOT(handleUpdateAction()));
   connect(installAction, SIGNAL(triggered()), 
           this, SLOT(handleInstallAction()));
   connect(removeAction, SIGNAL(triggered()), 
           this, SLOT(handleRemoveAction()));
   connect(noneAction, SIGNAL(triggered()), 
           this, SLOT(handleNoneAction()));
}

void  PackageTable::ActionContextMenu::exec(QModelIndex    ix,
                                            const QPoint&  pos)
{
   this->ix = ix;

   Package::State  state = model->getState(ix.row());

   switch (state) {
   case Package::OUT_OF_DATE: 
      installAction->setEnabled(false);
      updateAction->setEnabled(true);
      removeAction->setEnabled(true);
      break;
   case Package::NOT_INSTALLED:
      installAction->setEnabled(true);
      updateAction->setEnabled(false);
      removeAction->setEnabled(false);
      break;
   case Package::INSTALLED:
      installAction->setEnabled(false);
      updateAction->setEnabled(false);
      removeAction->setEnabled(true);
      break;
   }

   QMenu::exec(pos);
}

void  PackageTable::ActionContextMenu::handleUpdateAction()
{
   model->setAction(ix, Package::UPDATE);
}

void  PackageTable::ActionContextMenu::handleInstallAction()
{
   model->setAction(ix, Package::INSTALL);
}

void  PackageTable::ActionContextMenu::handleRemoveAction()
{
   model->setAction(ix, Package::REMOVE);
}

void  PackageTable::ActionContextMenu::handleNoneAction()
{
   model->setAction(ix, Package::NONE);
}
