#include "PackageTableDescribeDialog.h"
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QGridLayout>
#include <QtGui/QPushButton>
#include "HSeparator.h"
#include <QtGui/QLabel>
#include "Fonts.h"
#include <QtGui/QtEvents>

PackageTable::Describe::Dialog::Dialog(QWidget*  parent)
   : QDialog(parent),
     descrWidget(NULL),
     ready(true)
{
   setWindowTitle(tr("NUU Package Description"));

   setAttribute(Qt::WA_ShowModal, true);

   QVBoxLayout*  mainLayout  = new QVBoxLayout;
                 descrLayout = new QHBoxLayout;
   mainLayout->addLayout(descrLayout);

   QHBoxLayout*  buttonsLayout = new QHBoxLayout;
                 closeButton   = new QPushButton(tr("Close"), this);
   buttonsLayout->addStretch(1);
   buttonsLayout->addWidget(closeButton);
   connect(closeButton, SIGNAL(clicked()), this, SLOT(accept()));
   mainLayout->addLayout(buttonsLayout);

   setLayout(mainLayout);
}

void PackageTable::Describe::Dialog::config(YumIO*                 yumIO,
                                            const Package::Entry&  entry,
                                            YumConfIO*             yumConfIO)
{
                 descrWidget = new QWidget(this);
   QVBoxLayout*  layout      = new QVBoxLayout;

   QLabel*  headerLabel = new QLabel(yumIO->getName(entry), descrWidget);
   headerLabel->setFont(Fonts::makeHeader(headerLabel->font()));
   headerLabel->setAlignment(Qt::AlignHCenter);
   layout->addWidget(headerLabel);

   layout->addWidget(new HSeparator(descrWidget));

   instances.append(new Instance::Area(this, layout,
                                       yumIO, entry, yumConfIO, 
                                       /* primary => */ true));

   Package::Entry*  extra = entry.next;
   for (; extra != NULL; extra = extra->next) {
      layout->addWidget(new HSeparator(descrWidget));
      
      instances.append(new Instance::Area(this, layout,
                                          yumIO, *extra, yumConfIO,
                                          /* primary => */ false));
   }
   
   descrWidget->setLayout(layout);
   descrLayout->addWidget(descrWidget);
}

void PackageTable::Describe::Dialog::setVisible(bool  visible)
{
   if (!visible) {
      descrLayout->removeWidget(descrWidget);
      delete descrWidget;
      descrWidget = NULL;
      qDeleteAll(instances.begin(), instances.end());
      instances.clear();
   }

   QDialog::setVisible(visible);
}

void PackageTable::Describe::Dialog::setReady(bool  ready)
{
   this->ready = ready;

   closeButton->setEnabled(ready);
}

void  PackageTable::Describe::Dialog::done (int  r)
{
   if (ready) {
      QDialog::done(r);
   }
}

void  PackageTable::Describe::Dialog::closeEvent (QCloseEvent*  event)
{
   if (ready) {
      QDialog::closeEvent(event);
   } else {
      event->ignore();
   }
}
