#include "PackageTableDescribeInstanceArea.h"
#include "PackageTableDescribeDialog.h"
#include <QtGui/QHBoxLayout>
#include <QtGui/QLabel>
#include "Fonts.h"

#define SCROLL_AREA_DIMS  80, 24

PackageTable::Describe::Instance::Area::Area 
   (Dialog*                dialog,
    QVBoxLayout*           outerLayout,
    YumIO*                 yumIO,
    const Package::Entry&  entry,
    YumConfIO*             yumConfIO,
    bool                   primary)
   : dialog(dialog),
     yumIO(yumIO),
     entry(entry),
     yumConfIO(yumConfIO),
     parentWidget(outerLayout->widget()),
     repoDialog(NULL),
     textDialog(NULL)
{
                 scrollArea = new HScrollArea(SCROLL_AREA_DIMS, parentWidget);
                 widget     = new QWidget(scrollArea);
   QHBoxLayout*  hLayout    = new QHBoxLayout;
                 layout     = new QGridLayout;

   scrollArea->setFrameStyle(QFrame::NoFrame | QFrame::Plain);

   int  row = 0;

   addHeader(row, yumIO->getState(entry));

   switch (entry.kind) {
   case Package::SUITE:
   case Package::PACKAGE:
      if (primary) {
         addPair(row, tr("Kind"),         yumIO->getKind(entry));
         addPair(row, tr("Version"),      yumIO->getVerRel(entry));

         if (yumIO->getRepo(entry) != "installed") {
            Button  repo = addPairButton(row, tr("Repository"), 
                                         yumIO->getRepo(entry), tr("Details"));
            if (repo.button != NULL) {
               connect(repo.button, SIGNAL(clicked()),
                       this,        SLOT(handleRepo()));
            }
         }

         addPair(row, tr("Architecture"), yumIO->getArch(entry));
         addPair(row, tr("Size"),         yumIO->getSize(entry));
         addPair(row, tr("Summary"),      yumIO->getSummary(entry));
         addPair(row, tr("Description"),  yumIO->getDescription(entry));
         addPair(row, tr("Group"),        yumIO->getGroup(entry));
         addPair(row, tr("Vendor"),       yumIO->getVendor(entry));
         addPair(row, tr("Build Time"),   yumIO->getBuildTime(entry));
         addPair(row, tr("Build Host"),   yumIO->getBuildHost(entry));
         addPair(row, tr("Install Time"), yumIO->getInstallTime(entry));
         addPair(row, tr("Binary RPM"),   yumIO->getBinaryRpm(entry));
         addPair(row, tr("Source RPM"),   yumIO->getBinaryRpm(entry));
         addPair(row, tr("License"),      yumIO->getLicense(entry));
         addPair(row, tr("Packager"),     yumIO->getPackager(entry));
         addPair(row, tr("URL"),          yumIO->getUrl(entry));
         addPair(row, tr("Relocations"),  yumIO->getRelocations(entry));
         addPair(row, tr("Provides"),     yumIO->getProvides(entry));
         addPair(row, tr("Requires"),     yumIO->getRequires(entry));
         addPair(row, tr("Conflicts"),    yumIO->getConflicts(entry));
         addPair(row, tr("Obsoletes"),    yumIO->getObsoletes(entry));
      } else {
         addPair(row, tr("Version"),      yumIO->getVerRel(entry));
         addPair(row, tr("Size"),         yumIO->getSize(entry));
         addPair(row, tr("Build Time"),   yumIO->getBuildTime(entry));
         addPair(row, tr("Packager"),     yumIO->getPackager(entry));
      }

      files = addButton(row, tr("Files"), tr("Show"));
      connect(files.button, SIGNAL(clicked()),
              this,         SLOT(handleFiles()));

      changeLog = addButton(row, tr("Change Log"), tr("Show"));
      connect(changeLog.button, SIGNAL(clicked()),
              this,             SLOT(handleChangeLog()));
      break;
   case Package::GROUP:
      // Should always be primary
      addPair(row, tr("Kind"),              yumIO->getKind(entry));
      addPair(row, tr("Required Groups"),   yumIO->getRequiredGroups(entry));
      addPair(row, tr("Default MetaPkgs"),  yumIO->getDefaultMetaPkgs(entry));
      addPair(row, tr("Optional MetaPkgs"), yumIO->getOptionalMetaPkgs(entry));
      addPair(row, tr("Mandatory Packages"),   yumIO->getMandatoryPkgs(entry));
      addPair(row, tr("Default Packages"),     yumIO->getDefaultPkgs(entry));
      addPair(row, tr("Optional Packages"),    yumIO->getOptionalPkgs(entry));
      addPair(row, tr("Conditional Packages"), 
              yumIO->getConditionalPkgs(entry));
      break;
   }

   updateButtons();

   hLayout->addLayout(layout);
   hLayout->addStretch();

   widget->setLayout(hLayout);
   scrollArea->setWidget(widget);
   outerLayout->addWidget(scrollArea);
}

PackageTable::Describe::Instance::Area::~Area()
{
   if (repoDialog != NULL) delete repoDialog;
   if (textDialog != NULL) delete textDialog;
}

void  PackageTable::Describe::Instance::Area::addHeader (int&     row,
                                                         QString  text)
{
   QLabel*  label = new QLabel(text, widget);
   label->setFont(Fonts::makeHeader(label->font()));
   layout->addWidget(label, row, 0, 1, 2);

   row++;
}

void  PackageTable::Describe::Instance::Area::addPair (int&     row,
                                                       QString  key,
                                                       QString  value)
{
   if (value.isEmpty()) return;

   QLabel*  keyLabel   = new QLabel(tr("%1:").arg(key), widget);
   QLabel*  valueLabel = new QLabel(value,              widget);
   layout->addWidget(keyLabel,   row, 0, Qt::AlignRight | Qt::AlignTop);
   layout->addWidget(valueLabel, row, 1, Qt::AlignLeft  | Qt::AlignTop);
   
   row++;
}

PackageTable::Describe::Instance::Area::Button
   PackageTable::Describe::Instance::Area::addButton (int&     row,
                                                      QString  key,
                                                      QString  text)
{
   QLabel*       label  = new QLabel(tr("%1:").arg(key), widget);
   QPushButton*  button = new QPushButton(text, widget);
   layout->addWidget(label,  row, 0, Qt::AlignRight | Qt::AlignTop);
   layout->addWidget(button, row, 1, Qt::AlignLeft  | Qt::AlignTop);

   Button  result;
   result.text   = text;
   result.button = button;
   result.row    = row;

   row++;

   return result;
}

PackageTable::Describe::Instance::Area::Button
   PackageTable::Describe::Instance::Area::addPairButton (int&     row,
                                                          QString  key,
                                                          QString  value,
                                                          QString  text)
{
   if (value.isEmpty()) return Button();

   QLabel*       keyLabel    = new QLabel(tr("%1:").arg(key), widget);
   QHBoxLayout*  valueLayout = new QHBoxLayout();
   QLabel*       valueLabel  = new QLabel(value, widget);
   QPushButton*  button      = new QPushButton(text, widget);
   valueLayout->addWidget(valueLabel, 0, Qt::AlignTop);
   valueLayout->addWidget(button,     0, Qt::AlignTop);
   layout->addWidget(keyLabel,    row, 0, Qt::AlignRight | Qt::AlignTop);
   layout->addLayout(valueLayout, row, 1, Qt::AlignLeft  | Qt::AlignTop);

   Button  result;
   result.text   = text;
   result.button = button;
   result.row    = row;

   row++;

   return result;
}

void PackageTable::Describe::Instance::Area::updateButtons()
{
   switch (entry.kind) {
   case Package::SUITE:
   case Package::PACKAGE:
      updateButton(files,     yumIO->collectedFileLists);
      updateButton(changeLog, yumIO->collectedOtherData);
      break;
   case Package::GROUP:
      // DO NOTHING
      break;
   }
}

void  PackageTable::Describe::Instance::Area::updateButton (Button&  button,
                                                            bool     collected)
{
   if (button.button != NULL) {
      if (entry.state == Package::INSTALLED || collected) {
         button.button->setText(button.text);
      } else {
         button.button->setText(tr("%1 (must load extra data)")
                                   .arg(button.text));
      }
   }
}

void PackageTable::Describe::Instance::Area::handleRepo()
{
   Repo::EntryList::const_iterator  it = yumConfIO->entries.begin();
   for (; it != yumConfIO->entries.end(); ++it) {
      if (it->id == yumIO->getRepo(entry)) break;
   }
      
   if (repoDialog == NULL) {
      repoDialog = new RepoDialog(yumIO, parentWidget);
   }

   repoDialog->config(*it);
   repoDialog->exec();
}

void PackageTable::Describe::Instance::Area::handleFiles()
{
   dialog->setReady(false);

   QString  value = yumIO->getFiles(entry);

   updateButtons();

#if 0
   // This implementation is "cool".  It expands the file list in-line
   // instead of in a separate window.  But unfortunately a very long file
   // list will cause Qt to go totally draw-crazy and the dialog will become
   // unbearably slow.  I don't think it's because of the tricky code here.
   // I gather it's just too much content for the QLabel or QGridLayout or
   // QScrollableArea or somesuch.  So don't do this.
   QLabel*  valueLabel = new QLabel(value, widget);
   layout->removeWidget(files.button);
   layout->addWidget(valueLabel, files.row, 1);
   delete files.button;
   files.button = NULL;

   // Force Qt to resize the widget and its QGridLayout under the scrollArea
   // and to update the scrollArea's scrollbars if necessary.  This is the
   // minimum of hoop-jumping that forces Qt to get it right.
   widget->updateGeometry();
   scrollArea->forceResize();

   // The idea is to show all of the valueLabel if possible.  But if not, to
   // show as much as possible such that it includes the start of the label.
   // So, first ensure that the last pixel is visible, which may result in
   // scrolling down.  Then, ensure that the first pixel is visible, which
   // may result in scrolling up (but not necessarily as high as it was
   // before doing the two operations).
   scrollArea->ensureVisible(0, 
                             valueLabel->y() + valueLabel->size().height() - 1,
                             0, 
                             0);
   scrollArea->ensureVisible(0, 
                             valueLabel->y(), 
                             0, 
                             0);
#else
   if (textDialog == NULL) {
      textDialog = new TextDialog(parentWidget);
   }

   textDialog->setWindowTitle(tr("%1 %2 File List")
                                 .arg(yumIO->getName(entry))
                                 .arg(yumIO->getVerRel(entry)));
   textDialog->setText(value);
   textDialog->exec();
#endif

   dialog->setReady(true);
}

void PackageTable::Describe::Instance::Area::handleChangeLog()
{
   dialog->setReady(false);

   QString  value = yumIO->getChangeLog(entry);
   
   updateButtons();

   if (textDialog == NULL) {
      textDialog = new TextDialog(parentWidget);
   }

   textDialog->setWindowTitle(tr("%1 %2 Change Log")
                                 .arg(yumIO->getName(entry))
                                 .arg(yumIO->getVerRel(entry)));
   textDialog->setText(value);
   textDialog->exec();

   dialog->setReady(true);
}
