#include "PackageTableDescribeInstanceRepoDialog.h"
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QGridLayout>
#include <QtGui/QPushButton>
#include "HSeparator.h"
#include <QtGui/QLabel>
#include "Fonts.h"
#include <QtGui/QPushButton>
#include "RepoBrowse.h"

PackageTable::Describe::Instance::RepoDialog::RepoDialog(YumIO*    yumIO,
                                                         QWidget*  parent)
   : QDialog(parent),
     yumIO(yumIO),
     descrWidget(NULL)
{
   setWindowTitle(tr("NUU Repository Description"));

   setAttribute(Qt::WA_ShowModal, true);

   QVBoxLayout*  mainLayout  = new QVBoxLayout;
                 descrLayout = new QHBoxLayout;
   mainLayout->addLayout(descrLayout);

   QHBoxLayout*  buttonsLayout = new QHBoxLayout;
   QPushButton*  browseButton  = new QPushButton(tr("Browse Repository"), this);
   QPushButton*  closeButton   = new QPushButton(tr("Close"), this);
   browseButton->setToolTip(tr("Browse the first URL specified for the selected"
                               " repository using an external web browser"));
   buttonsLayout->addWidget(browseButton);
   buttonsLayout->addStretch(1);
   buttonsLayout->addWidget(closeButton);
   connect(browseButton, SIGNAL(clicked()), this, SLOT(browse()));
   connect(closeButton,  SIGNAL(clicked()), this, SLOT(accept()));
   mainLayout->addLayout(buttonsLayout);

   setLayout(mainLayout);
}

void PackageTable::Describe::Instance::RepoDialog::config
   (const Repo::Entry&  entry)
{
                 descrWidget = new QWidget(this);
   QVBoxLayout*  layout      = new QVBoxLayout;

   QLabel*  headerLabel = new QLabel(entry.id, descrWidget);
   headerLabel->setFont(Fonts::makeHeader(headerLabel->font()));
   headerLabel->setAlignment(Qt::AlignHCenter);
   layout->addWidget(headerLabel);

   layout->addWidget(new HSeparator(descrWidget));

   gridLayout = new QGridLayout;

   int  row = 0;

   addPair(row, tr("Name"),            entry.name);
   addPair(row, tr("URLs"),            entry.urls);
   addPair(row, tr("Mirror list URL"), entry.mirrorListUrl);
         
   layout->addLayout(gridLayout);
   descrWidget->setLayout(layout);
   descrLayout->addWidget(descrWidget);

   this->entry = &entry;
}

void PackageTable::Describe::Instance::RepoDialog::setVisible(bool visible)
{
   if (!visible) {
      descrLayout->removeWidget(descrWidget);
      delete descrWidget;
      descrWidget = NULL;
   }

   QDialog::setVisible(visible);
}

void  PackageTable::Describe::Instance::RepoDialog::addPair (int&     row,
                                                             QString  key,
                                                             QString  value)
{
   if (value.isEmpty()) return;

   QLabel*  keyLabel   = new QLabel(tr("%1:").arg(key), descrWidget);
   QLabel*  valueLabel = new QLabel(value,              descrWidget);
   gridLayout->addWidget(keyLabel,   row, 0, Qt::AlignRight | Qt::AlignTop);
   gridLayout->addWidget(valueLabel, row, 1, Qt::AlignLeft  | Qt::AlignTop);
   
   row++;
}

void PackageTable::Describe::Instance::RepoDialog::browse()
{
   RepoBrowse::browse(this, *entry, yumIO);
}
