#ifndef PackageTableModel_h
#define PackageTableModel_h

#include "YumIO.h"
#include <QtCore/QAbstractItemModel>
#include <QtGui/QItemSelectionModel>
#include <QtGui/QIcon>
#include <QtGui/QColor>
#include "YumConfIO.h"
#include "Package.h"
#include "PackageTableDescribeDialog.h"

namespace PackageTable {

   class Model : public QAbstractTableModel {
      Q_OBJECT
   public:
      explicit Model(YumIO*    yumIO,
                     QObject*  parent = NULL);
      ~Model();
   public:
      // Override methods from QAbstractTableModel
      int       rowCount    (const QModelIndex&  parent)       const;
      int       columnCount (const QModelIndex&  parent)       const;
      QVariant  data        (const QModelIndex&  ix, 
                             int                 role)         const;
      QVariant  headerData  (int                 section, 
                             Qt::Orientation     orientation, 
                             int                 role)         const;
      // New methods
      void            setYumConfIO (YumConfIO*          yumConfIO);
      Package::State  getState     (int                 row);
      void            setAction    (const QModelIndex&  ix,
                                    Package::Action     action);
      void            toggleAction (const QModelIndex&  ix);
      void            reset        ();
      void            describe     (const QModelIndex&  ix,
                                    QWidget*            parent);
      void            browse       (const QModelIndex&  ix,
                                    QWidget*            parent);
      void            setEnabled   (bool                enabled);
   public slots:
      // New methods called from MainWindow
      void  setSelected(const QModelIndexList&  indices);
      void  setUpdateAll();
      void  clearActions();
   protected:
      YumIO*             yumIO;
      YumConfIO*         yumConfIO;
      Describe::Dialog*  describeDialog;
      bool               enabled;
   protected:
      QIcon   outOfDateIcon;
      QIcon   upToDateIcon;
      QIcon   notInstalledIcon;

      QIcon   noneIcon;
      QIcon   updateIcon;
      QIcon   installIcon;
      QIcon   removeIcon;

      QIcon   suiteIcon;
      QIcon   groupIcon;
      QIcon   packageIcon;

      QColor  outOfDateColor;
      QColor  outOfDateInvColor;
      QColor  upToDateColor;
      QColor  upToDateInvColor;
      QColor  notInstalledColor;
      QColor  notInstalledInvColor;
   };

};

#endif
