#include "PasswdLineEdit.h"
#include <QtGui/QFont>
#include <QtGui/QStyle>
#include "Fonts.h"

PasswdLineEdit::PasswdLineEdit(const QRegExp&  regexp,
                               QWidget*        parent)
   : QLineEdit(parent),
     regexp(regexp),
     real(new QLineEdit(parent))
{
   setFont(Fonts::makeFixed(font()));

   connect(real, SIGNAL(textChanged(QString)),
           this, SLOT(textChanged(QString)));
   real->setFont(font());
   real->setGeometry(geometry());

   setAcceptDrops(false);
}

bool PasswdLineEdit::event(QEvent*  ev)
{
   bool  result = QLineEdit::event(ev);
   switch (ev->type()) {
   case QEvent::Move:
   case QEvent::Resize:
   case QEvent::KeyPress:
   case QEvent::KeyRelease:
   case QEvent::MouseMove:
   case QEvent::MouseButtonPress:
   case QEvent::MouseButtonRelease:
   case QEvent::MouseButtonDblClick:
      ((QLineEdit*)real)->event(ev);
      break;
   default:
      break;
   }
   return result;
}

void PasswdLineEdit::setRealText(const QString&  str)
{
   return real->setText(str);
}

QString PasswdLineEdit::realText() const
{
   return real->text();
}

void PasswdLineEdit::contextMenuEvent(QContextMenuEvent*)
{
   // Disable this.  It's not worth the effort to make it work with the
   // real QLineEdit.
}

void PasswdLineEdit::resizeEvent(QResizeEvent*  ev)
{
   QLineEdit::resizeEvent(ev);
   real->setGeometry(geometry());
}

void PasswdLineEdit::textChanged(const QString&  str)
{
   QString hidden = str;
   if (regexp.exactMatch(str)) {
      int    size       = regexp.cap(1).size();
      QChar  passwdChar
         = style()->styleHint(QStyle::SH_LineEdit_PasswordCharacter,
                              0, this);
      hidden.replace(regexp.pos(1), size, QString(size, passwdChar));
   }
   const int pos = real->cursorPosition();
   QLineEdit::setText(hidden);
   setCursorPosition(pos);
   real->setCursorPosition(pos);
   emit realTextChanged(real->text());
}
