#include <QtGui/QLineEdit>
#include <QtCore/QString>
#include <QtCore/QRegExp>
#include <QtGui/QResizeEvent>
#include <QtGui/QContextMenuEvent>

// QLineEdit with this extension:
//    For a string which matches the given REGEXP, the first capture is 
//    replaced with password-hiding characters (usually asterisks) when
//    displaying the string in the widget.
//    Some caveats:
//     * The font must be a fixed-width font.  (It ensures this when created,
//       but don't change it to a variable-width font after that.)
//     * setRealText     should be used in place of setText.
//     * realText        should be used in place of text.
//     * realTextChanged should be used in place of textChanged.
//     * The context menu is disabled.
//     * Be wary of any set* method.  Most would need some work to be supported.
class PasswdLineEdit : public QLineEdit
{
   Q_OBJECT
public:
   explicit PasswdLineEdit(const QRegExp&  regexp,
                           QWidget*        parent = NULL);
   void setRealText(const QString&  str);
   QString realText() const;
public:
   bool event(QEvent*  ev);
protected:
   void contextMenuEvent(QContextMenuEvent*);
   void resizeEvent(QResizeEvent*  ev);
protected slots:
   void textChanged(const QString&  str);
signals:
   void realTextChanged(const QString&);
protected:
   QRegExp     regexp;
   QLineEdit*  real;
};
