#include "PasswdTextEdit.h"
#include <QtGui/QFont>
#include <QtGui/QStyle>
#include <QtGui/QTextCursor>
#include "Fonts.h"

PasswdTextEdit::PasswdTextEdit(const QRegExp&  regexp,
                               QWidget*        parent)
   : HTextEdit(parent),
     regexp(regexp),
     real(new HTextEdit(parent))
{
   init();
}

PasswdTextEdit::PasswdTextEdit(const QRegExp&  regexp,
                               int             columns,
                               int             lines,
                               QWidget*        parent)
   : HTextEdit(columns, lines, parent),
     regexp(regexp),
     real(new HTextEdit(columns, lines, parent))
{
   init();
}

void PasswdTextEdit::init()
{
   setFont(Fonts::makeFixed(font()));

   connect(real, SIGNAL(textChanged()),
           this, SLOT(textChanged()));
   real->setFont(font());
   real->setGeometry(geometry());
   // This is a stunt to cause real's QAbstractScrollAreaPrivate::layoutChildren
   // to be called, which is the only way I could find to update real's 
   // viewport's geometry.
   ((PasswdTextEdit*)real)->setViewportMargins(0, 0, 0, 0);

   setAcceptDrops(false);
}

bool PasswdTextEdit::event(QEvent*  ev)
{
   bool  result = HTextEdit::event(ev);
   switch (ev->type()) {
   case QEvent::Move:
   case QEvent::Resize:
   case QEvent::KeyPress:
   case QEvent::KeyRelease:
   case QEvent::MouseMove:
   case QEvent::MouseButtonPress:
   case QEvent::MouseButtonRelease:
   case QEvent::MouseButtonDblClick:
      ((QObject*)real)->event(ev);
      break;
   default:
      break;
   }
   return result;
}

bool PasswdTextEdit::viewportEvent(QEvent*  ev)
{
   bool  result = HTextEdit::viewportEvent(ev);
   switch (ev->type()) {
   case QEvent::Move:
   case QEvent::Resize:
   case QEvent::KeyPress:
   case QEvent::KeyRelease:
   case QEvent::MouseMove:
   case QEvent::MouseButtonPress:
   case QEvent::MouseButtonRelease:
   case QEvent::MouseButtonDblClick:
      // This technically is wrong because real isn't a PasswdTextEdit,
      // but it works around the fact that HTextEdit::viewportEvent is
      // protected, and ends up using the same vtable slot, so it works.
      ((PasswdTextEdit*)real)->viewportEvent(ev);
      break;
   default:
      break;
   }
   return result;
}

void PasswdTextEdit::setRealPlainText(const QString&  str)
{
   return real->setPlainText(str);
}

QString PasswdTextEdit::realPlainText() const
{
   return real->toPlainText();
}

void PasswdTextEdit::contextMenuEvent(QContextMenuEvent*)
{
   // Disable this.  It's not worth the effort to make it work with the
   // real HTextEdit.
}

void PasswdTextEdit::resizeEvent(QResizeEvent*  ev)
{
   HTextEdit::resizeEvent(ev);
   real->setGeometry(geometry());
   // This is a stunt to cause real's QAbstractScrollAreaPrivate::layoutChildren
   // to be called, which is the only way I could find to update real's 
   // viewport's geometry.
   ((PasswdTextEdit*)real)->setViewportMargins(0, 0, 0, 0);
}

void PasswdTextEdit::textChanged()
{
   QString hidden = real->toPlainText();
   int     ix     = 0;
   for (;;) {
      ix = regexp.indexIn(hidden, ix);
      if (ix == -1) break;

      ix += regexp.matchedLength();

      int    size       = regexp.cap(1).size();
      QChar  passwdChar
         = style()->styleHint(QStyle::SH_LineEdit_PasswordCharacter,
                              0, this);
      hidden.replace(regexp.pos(1), size, QString(size, passwdChar));
   }

   int pos = real->textCursor().position();
   HTextEdit::setPlainText(hidden);
   QTextCursor  cursor = textCursor();
   cursor.setPosition(pos);
   setTextCursor(cursor);
   QTextCursor  realCursor = real->textCursor();
   realCursor.setPosition(pos);
   real->setTextCursor(realCursor);
   emit realTextChanged();
}
