#include "HTextEdit.h"
#include <QtCore/QString>
#include <QtCore/QRegExp>
#include <QtGui/QResizeEvent>
#include <QtGui/QContextMenuEvent>

// HTextEdit with this extension:
//    For a substring which matches the given REGEXP, the first capture is 
//    replaced with password-hiding characters (usually asterisks) when
//    displaying the string in the widget.
//    Some caveats:
//     * The font must be a fixed-width font.  (It ensures this when created,
//       but don't change it to a variable-width font after that.)
//     * setRealPlainText should be used in place of setPlainText.
//     * realPlainText    should be used in place of toPlainText.
//     * realTextChanged  should be used in place of textChanged.
//     * The context menu is disabled.
//     * Be wary of any set* method.  Most would need some work to be supported.
class PasswdTextEdit : public HTextEdit
{
   Q_OBJECT
public:
   explicit PasswdTextEdit(const QRegExp&  regexp,
                           QWidget*        parent = NULL);
   explicit PasswdTextEdit(const QRegExp&  regexp,
                           int             columns,
                           int             lines,
                           QWidget*        parent = NULL);
   void setRealPlainText(const QString&  str);
   QString realPlainText() const;
protected:
   bool event(QEvent*  ev);
   bool viewportEvent(QEvent*  ev);
   void contextMenuEvent(QContextMenuEvent*);
   void resizeEvent(QResizeEvent*  ev);
protected:
   void init();
protected slots:
   void textChanged();
signals:
   void realTextChanged();
protected:
   QRegExp     regexp;
   HTextEdit*  real;
};
