#include "RepoAuthChecker.h"
#include "RepoBuffet.h"
#include "URL.h"

RepoAuth::Checker::Checker(QWidget*    parent,
                           YumConfIO*  yumConfIO)
   : quit(false),
     changed(false),
     dialog(NULL)
{
   QHash<QString,Auth>  applyHost;

   bool   shown    = false;
   Auth*  applyAll = NULL;

   yumConfIO->read(parent);

   Repo::EntryList::iterator  it = yumConfIO->entries.begin();
   for (; it != yumConfIO->entries.end(); ++it) {
      if (yumConfIO->reposUsed[it->id]) {
         QString  nameKey;
         QString  name;
         QString  passwordKey;
         QString  password;
         if (Repo::Buffet::getAuth(*it, nameKey, name, passwordKey, password)) {
            if (name.isEmpty() && password.isEmpty()) {
               if (dialog == NULL) {
                  dialog = new RepoAuth::Dialog(parent);
               }
               QString  host = URL::getHost(it->urls);
               if (host.isEmpty()) {
                  host = URL::getHost(it->mirrorListUrl);
               }

               if (applyAll != NULL) {
                  replace(*it, *applyAll);
               } else if (applyHost.contains(host)) {
                  replace(*it, applyHost[host]);
               } else {
                  dialog->exec(&*it);
                  shown = true;
                  Dialog::Result  result = (Dialog::Result)dialog->result();
                  if (result == Dialog::QUIT) {
                     quit = true;
                     break;
                  } else if (result == Dialog::SKIP_ALL) {
                     break;
                  } else if (result == Dialog::DISABLE) {
                     yumConfIO->reposUsed[it->id] = false;
                  } else if (result == Dialog::OK
                             || result == Dialog::APPLY_ALL
                             || result == Dialog::APPLY_HOST) {
                     QString  newNameKey;
                     QString  newName;
                     QString  newPasswordKey;
                     QString  newPassword;
                     Repo::Buffet::getAuth(*it, 
                                           newNameKey,     newName, 
                                           newPasswordKey, newPassword);
                     if (newName != name || newPassword != password) {
                        changed = true;
                     }
                     if (result == Dialog::APPLY_ALL) {
                        applyAll = new Auth(newName, newPassword);
                     } else if (result == Dialog::APPLY_HOST) {
                        Auth&  auth = applyHost[host];
                        auth.name     = newName;
                        auth.password = newPassword;
                     }
                  }
               }
            }
         }
      }
   }

   if (shown) {
      dialog->setVisible(false);
   }

   if (applyAll != NULL) {
      delete applyAll;
   }

   if (changed) {
      yumConfIO->write(parent);
   }
}

RepoAuth::Checker::~Checker()
{
   if (dialog != NULL) {
      delete dialog;
   }
}

void RepoAuth::Checker::replace(Repo::Entry&  entry,
                                const Auth&   auth)
{
   entry.urls          = Repo::Buffet::replaceAuth(entry.urls,
                                                   auth.name,
                                                   auth.password);
   entry.mirrorListUrl = Repo::Buffet::replaceAuth(entry.mirrorListUrl,
                                                   auth.name, 
                                                   auth.password);
}
