#include "RepoAuthDialog.h"
#include "RepoBuffet.h"
#include <QtGui/QVBoxLayout>
#include "Fonts.h"
#include <QtGui/QGridLayout>
#include "HSeparator.h"
#include <QtGui/QHBoxLayout>
#include <QtGui/QStyle>
#include <QtGui/QStyleOptionFrame>

RepoAuth::Dialog::Dialog(QWidget*  parent)
   : QDialog(parent),
     entry(NULL)
{
   setWindowTitle(tr("NUU Authentication"));

   setAttribute(Qt::WA_ShowModal, true);

   QVBoxLayout*  mainLayout = new QVBoxLayout();

   QLabel*  headerLabel = new QLabel(tr("Some configured repositories require"
                                        " authentication"), this);
   headerLabel->setFont(Fonts::makeHeader(headerLabel->font()));
   headerLabel->setAlignment(Qt::AlignHCenter);
   mainLayout->addWidget(headerLabel);
   mainLayout->addWidget(new HSeparator(this));

   QGridLayout*  gutsLayout = new QGridLayout();

   QLabel*  idLabel = new QLabel(tr("Identifier"), this);
            idValue = new QLabel(this);
   gutsLayout->addWidget(idLabel, 0, 0);
   gutsLayout->addWidget(idValue, 0, 1);

   QLabel*  nameLabel  = new QLabel(tr("Name"), this);
            nameValue  = new QLabel(this);
   gutsLayout->addWidget(nameLabel, 1, 0);
   gutsLayout->addWidget(nameValue, 1, 1);

   QLabel*  urlsLabel = new QLabel(tr("URLs"), this);
            urlsValue = new QLabel(this);
   gutsLayout->addWidget(urlsLabel, 2, 0);
   gutsLayout->addWidget(urlsValue, 2, 1);

   QLabel*  mirrorListUrlLabel = new QLabel(tr("Mirror list URL"), this);
            mirrorListUrlValue = new QLabel(this);
   gutsLayout->addWidget(mirrorListUrlLabel, 3, 0);
   gutsLayout->addWidget(mirrorListUrlValue, 3, 1);

   gutsLayout->addWidget(new HSeparator(this), 4, 0, 1, 2);

            authNameLabel = new QLabel(tr("Name"), this);
            authNameEdit  = new QLineEdit(this);
   authNameEdit->setToolTip(tr("Authentication name for a repository using the"
                               " buffet authentication method"));
   connect(authNameEdit, SIGNAL(textEdited(const QString&)), 
           this,         SLOT(authChanged(const QString&)));
   gutsLayout->addWidget(authNameLabel, 5, 0);
   gutsLayout->addWidget(authNameEdit,  5, 1);

            authPasswordLabel = new QLabel(tr("Password"), this);
            authPasswordEdit  = new QLineEdit(this);
   authPasswordEdit->setEchoMode(QLineEdit::Password);
   authPasswordEdit->setToolTip(tr("Authentication password for a repository"
                                   " using the buffet authentication method"));
   connect(authPasswordEdit, SIGNAL(textEdited(const QString&)), 
           this,             SLOT(authChanged(const QString&)));
   gutsLayout->addWidget(authPasswordLabel, 6, 0);
   gutsLayout->addWidget(authPasswordEdit,  6, 1);

   mainLayout->addLayout(gutsLayout);

   QHBoxLayout*  buttonLayout    = new QHBoxLayout();
                 quitButton      = new QPushButton(tr("Quit"), this);
                 disableButton   = new QPushButton(tr("Disable for Session"),
                                                   this);
                 skipAllButton   = new QPushButton(tr("Skip All"), this);
                 applyAllButton  = new QPushButton(tr("Apply to All"), this);
                 applyHostButton = new QPushButton(tr("Apply to All with this"
                                                      " Host"), this);
                 okButton        = new QPushButton(tr("OK"), this);
   okButton->setDefault(true);
   connect(quitButton,      SIGNAL(clicked()), this, SLOT(handleQuit()));
   connect(disableButton,   SIGNAL(clicked()), this, SLOT(handleDisable()));
   connect(skipAllButton,   SIGNAL(clicked()), this, SLOT(handleSkipAll()));
   connect(applyAllButton,  SIGNAL(clicked()), this, SLOT(handleApplyAll()));
   connect(applyHostButton, SIGNAL(clicked()), this, SLOT(handleApplyHost())); 
   connect(okButton,        SIGNAL(clicked()), this, SLOT(handleOK()));
   buttonLayout->addStretch(1);
   buttonLayout->addWidget(quitButton);
   buttonLayout->addWidget(disableButton);
   buttonLayout->addWidget(skipAllButton);
   buttonLayout->addWidget(applyAllButton);
   buttonLayout->addWidget(applyHostButton);
   buttonLayout->addWidget(okButton);
   mainLayout->addLayout(buttonLayout);

   setLayout(mainLayout);   
}

void RepoAuth::Dialog::exec(Repo::Entry*  entry)
{
   this->entry = entry;

   idValue->setText(entry->id);
   nameValue->setText(entry->name);
   urlsValue->setText(entry->urls);
   mirrorListUrlValue->setText(entry->mirrorListUrl);

   QString  nameKey;
   QString  name;
   QString  passwordKey;
   QString  password;
   Repo::Buffet::getAuth(*entry, nameKey, name, passwordKey, password);
   authNameLabel->setText(nameKey);
   authNameEdit->setText(name);
   authPasswordLabel->setText(passwordKey);
   authPasswordEdit->setText(password);

   skipAllButton->setEnabled(true);
   quitButton->setEnabled(true);
   disableButton->setEnabled(true);
   okButton->setEnabled(true);

   authNameEdit->setFocus(Qt::OtherFocusReason);

   // Cannot call QDialog::exec() here because that uses its own
   // d->eventLoop.  And the only way to exit from that eventLoop is via
   // QDialog::setVisible(false).  But I want to exit this event loop and
   // still leave the window visible.  So do it manually.

   show();

   // Use modal event loop until eventLoop.exit() is called
   eventLoop.exec();
}


void RepoAuth::Dialog::done(int  r)
{
   skipAllButton->setEnabled(false);
   quitButton->setEnabled(false);
   disableButton->setEnabled(false);
   okButton->setEnabled(false);
   
   setResult(r);

   eventLoop.exit(); // Exit the modal eventLoop in exec()

   emit finished(r);
   emit finished(r);
   if (r == Accepted) {
      emit accepted();
   } else if (r == Rejected) {
      emit rejected();
   }
}

QSize RepoAuth::Dialog::sizeHint() const 
{
   // The default size almost certainly will be determined by the url or
   // mirrorListUrl labels.  And they are going to grow as the user types in
   // authentication information.  So add some extra space to the dialog so
   // that it doesn't need to grow to accomodate that.  This amount added is
   // a guess, so this is only a heuristic.

   QSize  size = QDialog::sizeHint();
   size.setWidth(size.width() + fontMetrics().width('n') * 40);
   return size;
}

void RepoAuth::Dialog::authChanged(const QString&  text)
{
   QString  nameKey     = authNameLabel->text();
   QString  name        = authNameEdit->text();
   QString  passwordKey = authPasswordLabel->text();
   QString  password    = authPasswordEdit->text();

   password.fill(style()->styleHint(QStyle::SH_LineEdit_PasswordCharacter,
                                    0, this));

   urlsValue->setText(Repo::Buffet::replaceAuth(urlsValue->text(),
                                                nameKey,     name, 
                                                passwordKey, password));
   mirrorListUrlValue->setText(Repo::Buffet::replaceAuth
                                  (mirrorListUrlValue->text(),
                                   nameKey,     name, 
                                   passwordKey, password));
}

void RepoAuth::Dialog::setUrls()
{
   QString  nameKey     = authNameLabel->text();
   QString  name        = authNameEdit->text();
   QString  passwordKey = authPasswordLabel->text();
   QString  password    = authPasswordEdit->text();

   entry->urls = Repo::Buffet::replaceAuth(urlsValue->text(),
                                           nameKey,     name, 
                                           passwordKey, password);
   entry->mirrorListUrl = Repo::Buffet::replaceAuth(mirrorListUrlValue->text(),
                                                    nameKey,     name, 
                                                    passwordKey, password);
}

void RepoAuth::Dialog::handleQuit()
{
   entry = NULL;
   done(QUIT);
}

void RepoAuth::Dialog::handleDisable()
{
   entry = NULL;
   done(DISABLE);
}

void RepoAuth::Dialog::handleSkipAll()
{
   entry = NULL;
   done(SKIP_ALL);
}

void RepoAuth::Dialog::handleApplyAll()
{
   setUrls();

   entry = NULL;
   done(APPLY_ALL);
}

void RepoAuth::Dialog::handleApplyHost()
{
   setUrls();

   entry = NULL;
   done(APPLY_HOST);
}

void RepoAuth::Dialog::handleOK()
{
   setUrls();

   entry = NULL;
   done(OK);
}
