#include "RepoBrowse.h"
#include "Share.h"
#include "Execute.h"
#include <QtGui/QMessageBox>

extern void RepoBrowse::browse (QWidget*            parent,
                                const Repo::Entry&  entry,
                                YumIO*              yumIO)
{
   QString  doc;

   if (!entry.urls.isEmpty()) {
      QRegExp  whitespace("\\s+");
      doc = entry.urls.section(whitespace, 0, 0, QString::SectionSkipEmpty);
   } else if (!entry.mirrorListUrl.isEmpty()) {
      doc = entry.mirrorListUrl;
   } else {
      QMessageBox::critical(/* parent  => */ parent,
                            /* caption => */ QObject::tr("Error"),
                            /* text    => */ QObject::tr("Repository"
                                                         " configuration has no"
                                                         " URL"));
   }

   {
      Python::PyGILStateWrapper  gsw;
      doc.replace("$basearch",   yumIO->getYumVar("basearch"));
      doc.replace("$arch",       yumIO->getYumVar("arch"));
      doc.replace("$releasever", yumIO->getYumVar("releasever"));
   }

   char*  docStr = new char[doc.length()+1];
   strcpy(docStr, doc.toStdString().c_str());

   QString  tool = Share::absolute("webbrowser");
   char*    toolStr = new char[tool.length()+1];
   strcpy(toolStr, tool.toStdString().c_str());

   char*  argv[3];
   int    arg = 0;
   argv[arg++] = toolStr;
   argv[arg++] = docStr;
   argv[arg++] = NULL;

   Execute::execute(parent, argv);

   delete [] toolStr;
   delete [] docStr;
}

extern void RepoBrowse::browse (QWidget*    parent,
                                QString     repoId,
                                YumIO*      yumIO,
                                YumConfIO*  yumConfIO)
{
   Repo::EntryList::const_iterator  it = yumConfIO->entries.begin();
   for (; it != yumConfIO->entries.end(); ++it) {
      if (it->id == repoId) break;
   }

   browse(parent, *it, yumIO);
}
