#ifndef RepoEditorConfig_h
#define RepoEditorConfig_h

#include <QtCore/QObject>
#include <QtGui/QWidget>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QLabel>
#include "PasswdLineEdit.h"
#include "PasswdTextEdit.h"
#include "HTextEdit.h"
#include <QtGui/QComboBox>
#include <QtGui/QSpinBox>
#include "Repo.h"
#include "YumConfIO.h"

namespace RepoEditor {

   // An editor to configure a single yum repository

   class Config : public QObject {
      Q_OBJECT
   public:
      explicit Config(QGridLayout*  parentLayout);
   public:
      void  setDefaults (bool  enabled,
                         bool  gpgcheck);
      void  disable();
   public:
      void         setEntry (const Repo::Entry&  entry);
      Repo::Entry  toEntry  ();
      bool         setStatus(YumConfIO*  yumConfIO,
                             QLabel*     label);
   signals:
      void  changed();
   protected slots:
      void authChanged  (const QString&  text);
      void urlChanged   ();
      void urlChanged   (const QString&  text);
      void valueChanged ();
      void valueChanged (int             state);
      void valueChanged (const QString&  text);
   protected:
      bool  programmaticChangeInProgress;
   protected:
      QGroupBox*  createAuth          (QWidget*  parent);
      QGroupBox*  createOptions       (QWidget*  parent);
      QLayout*    createSimpleOptions (QWidget*  parent);
      QGroupBox*  createFileInfo      (QWidget*  parent);
   public:
      QLineEdit*       idEdit;
   protected:
      QLineEdit*       nameEdit;
      PasswdTextEdit*  urlsEdit;
      PasswdLineEdit*  mirrorListUrlEdit;
      QLabel*          authNameLabel;
      QLineEdit*       authNameEdit;
      QLabel*          authPasswordLabel;
      QLineEdit*       authPasswordEdit;
      QComboBox*       enabledComboBox;
      QComboBox*       checkGPGKeysComboBox;
      HTextEdit*       otherOptionsEdit;
      QLineEdit*       filenameEdit;
      QDoubleSpinBox*  sortPositionSpinBox;
   };

};

#endif
