#ifndef REPO_EDITOR_ONE_DIALOG

#include "MainWindow.h"
#include "RepoEditorConfigDialog.h"
#include "RepoEditor.h"
#include "RepoBuffet.h"
#include <QtGui/QVBoxLayout>
#include <QtGui/QPushButton>
#include <QtGui/QLabel>
#include "Colors.h"
#include "RepoBrowse.h"

RepoEditor::ConfigDialog::ConfigDialog(YumConfIO*    yumConfIO,
                                       QWidget*      parent)
   : HDialog(parent),
     yumConfIO(yumConfIO),
     entry(entry)
{
   setWindowTitle(tr("NUU Repository Configuration"));

   setAttribute(Qt::WA_ShowModal, true);

   QVBoxLayout*  mainLayout = new QVBoxLayout(this);

   QGridLayout*  configLayout = new QGridLayout();
   config = new Config(configLayout);
   connect(config, SIGNAL(changed()), this, SLOT(changed()));
   config->disable();
   mainLayout->addLayout(configLayout);

   QHBoxLayout*  dialogButtonLayout = new QHBoxLayout();
                 statusLabel        = new QLabel("", this);
                 resetButton        = new QPushButton(tr("Reset"),     this);
   QPushButton*  cancelButton       = new QPushButton(tr("Cancel"),    this);
   QPushButton*  okButton           = new QPushButton(tr("OK"),        this);
   okButton->setDefault(true);
   statusLabelDefaultPalette = statusLabel->palette();
   resetButton->setToolTip(tr("Undo changes to this repository configuration"));
   cancelButton->setToolTip(tr("Undo changes to this repository configuration"
                               " and close the window"));
   okButton->setToolTip(tr("Apply changes to all repository configurations, and"
                           " close the window"));
   dialogButtonLayout->addWidget(statusLabel);
   dialogButtonLayout->addStretch(1);
   dialogButtonLayout->addWidget(resetButton);
   dialogButtonLayout->addWidget(cancelButton);
   dialogButtonLayout->addWidget(okButton);
   connect(resetButton,    SIGNAL(clicked()), this, SLOT(reset()));
   connect(cancelButton,   SIGNAL(clicked()), this, SLOT(cancel()));
   connect(okButton,       SIGNAL(clicked()), this, SLOT(ok()));

   mainLayout->addLayout(dialogButtonLayout);

   setLayout(mainLayout);   
}

void RepoEditor::ConfigDialog::setEntry(Repo::Entry*  entry)
{
   this->entry = entry;
   config->setEntry(*entry);
   updateStatusLabel();
}

void RepoEditor::ConfigDialog::setVisible(bool  visible)
{
   if (visible) {
      config->setDefaults(yumConfIO->enabled, 
                          yumConfIO->gpgcheck);
      setHasChanged(false);

      config->idEdit->setFocus(Qt::OtherFocusReason);
   }

   QDialog::setVisible(visible);
}

void RepoEditor::ConfigDialog::changed()
{
   setHasChanged(true);
}

void RepoEditor::ConfigDialog::reset()
{
   config->setEntry(*entry);
   setHasChanged(false);
}

void RepoEditor::ConfigDialog::cancel()
{
   reject();
}

void RepoEditor::ConfigDialog::ok()
{
   if (hasChanged) {
      Repo::Entry  newEntry = config->toEntry();

      yumConfIO->reposUsed[newEntry.id]
         = (newEntry.enabled == Repo::True  ? true  :
            newEntry.enabled == Repo::False ? false :
                                              yumConfIO->enabled);

      *entry = newEntry;

      accept();
   } else {
      reject();
   }
}

void RepoEditor::ConfigDialog::updateStatusLabel()
{
   // Display an errors or warnings about the currently selected repository
   // first.  This means we may see a warning about the currently selected
   // repository in preference to a general error related to the state of
   // all repositories.

   if (config->setStatus(yumConfIO, statusLabel)) {
      return;
   }

   statusLabel->setPalette(statusLabelDefaultPalette);
   statusLabel->setText("");
}

void RepoEditor::ConfigDialog::setHasChanged(bool value)
{
   hasChanged = value;
   resetButton->setEnabled(hasChanged);

   updateStatusLabel();
}

#endif
