#ifndef RepoEditorDialog_h
#define RepoEditorDialog_h

#include "YumIO.h"
#include "HDialog.h"
#include "RepoEditorConfig.h"
#include "RepoEditorConfigDialog.h"
#include "RepoEditorModel.h"
#include "RepoEditorView.h"
#include "YumConfIO.h"
#include <QtGui/QPushButton>
#include <QtGui/QLabel>
#include <QtGui/QPalette>
#include "YumConfIO.h"
#include "YumReposUsed.h"

namespace RepoEditor {

   // The repository editor dialog, for configuring the set of yum repositories

   // Avoid subclassing from QDialog because that comes with the "default
   // button" baggage which doesn't make any sense for this dialog.  Note
   // that keyPressEvent still supports using Escape to bail out of the
   // this dialog, though.
   class Dialog : public HDialog {
      Q_OBJECT
   public:
      explicit Dialog(YumIO*      yumIO,
                      YumConfIO*  yumConfIO,
                      QWidget*    parent);
   public slots:
      void setVisible(bool visible);
   signals:
      void reposChanged();
   protected slots:
      void currentChanged(int  currentRow,
                          int  previousRow);
      void useChanged();
      void configChanged();
#ifdef REPO_EDITOR_ONE_DIALOG
      void entryChanged();
#endif
      void browse();
#ifndef REPO_EDITOR_ONE_DIALOG
      void edit ();
      void edit (const QModelIndex&  ix);
#endif
      void add();
      void copy();
      void remove();
   protected slots:
      void resetAll();
      void reset();
      void cancel();
      bool apply();
      void ok();
   protected:
      bool     hasEmptyId();
      QString  getDuplicateId();
   protected:
      void updateStatusLabel   ();
      void setEntryToOldId     (QString  oldId);
      void setEntryHasChanged  (bool     value);
      void setConfigHasChanged (bool     value);
   public:
      bool  forceReposChanged;
   protected:
      QWidget*       parent;
      Model*         model;
      View*          view;
      QPushButton*   browseButton;
#ifndef REPO_EDITOR_ONE_DIALOG
      QPushButton*   editButton;
#endif
      QPushButton*   copyButton;
      QPushButton*   removeButton;
      QLabel*        statusLabel;
#ifdef REPO_EDITOR_ONE_DIALOG
      QPushButton*   resetButton;
#endif
      QPushButton*   resetAllButton;
      QPushButton*   applyButton;
#ifdef REPO_EDITOR_ONE_DIALOG
      Config*        config;
#else
      ConfigDialog*  configDialog;
#endif
   protected:
      QPalette  statusLabelDefaultPalette;
   protected:
      YumIO*         yumIO;
      YumConfIO*     yumConfIO;
      YumReposUsed   originalYumReposUsed;
   protected:
      bool  useHasChanged;    // any use checkbox has changed
      bool  entryHasChanged;  // the current entry has changed
      bool  configHasChanged; // any entry in the config has changed
                              // can be true only with REPO_EDITOR_ONE_DIALOG
      bool  applied;
   };

};

#endif
