#ifndef RepoEditorModel_h
#define RepoEditorModel_h

#include "YumIO.h"
#include <QtCore/QAbstractItemModel>
#include <QtGui/QFont>
#include <QtGui/QIcon>
#include "RepoEditor.h"
#include "YumConfIO.h"

namespace RepoEditor {

   class Model : public QAbstractTableModel {
      Q_OBJECT
   public:
      explicit Model(YumIO*      yumIO,
                     YumConfIO*  yumConfIO,
                     QObject*    parent = NULL);
      ~Model();
   public:
      void  setViewFont (const QFont&  font);
   public:
      // Override methods from QAbstractTableModel
      int            rowCount    (const QModelIndex&  parent)       const;
      int            columnCount (const QModelIndex&  parent)       const;
      QVariant       data        (const QModelIndex&  ix, 
                                  int                 role)         const;
      QVariant       headerData  (int                 section, 
                                  Qt::Orientation     orientation, 
                                  int                 role)         const;
   public:
      int          addRow        (const Repo::Entry&  entry);
      bool         removeRow     (int                 row,
                                  const QModelIndex&  parent = QModelIndex());
      void         reset         ();
      QModelIndex  createIndex   (int                 row);
      void         toggleUse     (const QModelIndex&  ix);
      void         toggleEnabled (const QModelIndex&  ix);
      void         browse        (const QModelIndex&  ix,
                                  QWidget*            parent);
   public:
      void  dataChanged (int  row);
   signals:
      void  useChanged();
      void  configChanged();
   protected:
      YumIO*      yumIO;
      YumConfIO*  yumConfIO;
      QFont       italicFont;
   protected:
      QIcon   noneBoxIcon;
      QIcon   checkBoxIcon;
   };

};

#endif
