#ifndef YumConfIO_h
#define YumConfIO_h

#undef DEBUG_YUMCONFIO

#include "Repo.h"
#include <QtCore/QObject>
#include <QtGui/QWidget>
#include <QtCore/QSet>
#include <QtCore/QString>
#include <string>
#include "YumReposEnabled.h"
#include "YumReposUsed.h"

class YumConfIO : public QObject {
   Q_OBJECT
public:
   explicit YumConfIO(const YumReposEnabled&  reposEnabled,
                      YumReposUsed&           reposUsed);
public:
   bool read  (QWidget*  errorParent);
   bool write (QWidget*  errorParent);
signals:
   void changed();
protected:
   enum Status {
      READ,
      END_OF_FILE,
      ERROR
   };
   bool         readConfFile     (FILE*         file);
   bool         readRepoFile     (FILE*         file,
                                  QString       pathname);
   Status       readLogicalLine  (FILE*         file,
                                  QString       pathname,
                                  QString&      line);
   Status       readLine         (FILE*         file,
                                  QString       pathname,
                                  QString&      line);
   void         unreadLine       (QString       line);
   static bool  isOption         (QString       line,
                                  QString       name,
                                  QString&      value);
   void         cleanupRead      (Repo::Entry&  entry);
   void         applyUsedOptions (Repo::Entry&  entry);
protected:
   bool         checkErrors         ();
   bool         hasEmptyId          ();
   QString      getDuplicateId      ();
   void         cleanupWrite        (Repo::EntryList&    e);
   static bool  filePositionLessThan(const Repo::Entry&  l, 
                                     const Repo::Entry&  r);
   bool         writeRepo           (FILE*               file,
                                     const Repo::Entry&  entry);
   bool         removeUnusedFiles   (QSet<QString>&      filenamesSeen);
public:
   QString                 cachedir;     // cachedir from /etc/yum.conf
   QString                 reposdir;     // reposdir from /etc/yum.conf
   bool                    enabled;      // enabled default from /etc/yum.conf
   bool                    gpgcheck;     // gpgcheck default from /etc/yum.conf
   Repo::EntryList         entries;     
   const YumReposEnabled&  reposEnabled;
   YumReposUsed&           reposUsed;
#ifdef DEBUG_YUMCONFIO
public:
   void dump();
#endif
protected:
   QString          options;      // [main] options from /etc/yum.conf
protected:
   void error(QString  text);
protected:
   QWidget*  errorParent;
protected:
   QList<QString>  readAhead;
};

#define YUM_CONF  "/etc/yum.conf"

#endif
