#ifndef Yum_h
#define Yum_h

/*
** DANGER DANGER!  If you #include this, it must be #include'd first.  This is
** so that "Python.h" will be included first and so that its <Python.h> will be
** included first.  Python obnoxiously demands that that header be included
** first, and things sometimes go awry if it isn't.  (For example, you may get
** duplicate definitions of _POSIX_C_SOURCE.)
*/

#include "Python.h"
#include "Package.h"
#include "Log.h"
#include <QtGui/QWidget>
#include <QtCore/QHash>
#include <QtCore/QList>
#include <QtCore/QString>
#include <QtCore/QSet>
#include <QtCore/QEventLoop>
#include "Progress.h"
#include "YumIODialogs.h"
#include "YumIOCapturers.h"
#include "YumReposUsed.h"

class CollectThread;
class ApplyThread;
class CollectFileListsThread;
class CollectOtherDataThread;

class YumIO : public QObject, private Python {
   Q_OBJECT
public:
   explicit YumIO();
   ~YumIO();
public:
   void  setup(QWidget*       parent,
               HProgressBar*  totalProgressBar,
               HProgressBar*  itemProgressBar);
public:
   bool  initialize        ();
   bool  collect           (const YumReposUsed&  reposUsed);
   bool  collectFileLists  ();
   bool  collectOtherData  ();
   void  clear             ();
   bool  apply             ();
public:
   bool  cleanHeaders      ();
   bool  cleanPackages     ();
   bool  cleanXmlMetadata  ();
   bool  cleanPickledCache ();
   bool  cleanAll          ();
public:
   inline int  count() const 
   { return list.count(); }
   inline Package::EntryList::const_iterator  begin() const
   { return list.begin(); }
   inline Package::EntryList::const_iterator  end() const 
   { return list.end(); }
   inline Package::Entry* const&  operator[](int i) const
   { return list[i]; }
public:
   QString  getState            (const Package::Entry&  entry);
   QString  getKind             (const Package::Entry&  entry);
   // For packages & suites
   QString  getName             (const Package::Entry&  entry)
      { return entry.name; }
   QString  getVerRel           (const Package::Entry&  entry)
      { return entry.verrel; }
   QString  getRepo             (const Package::Entry&  entry)
      { return entry.repo; }
   QString  getArch             (const Package::Entry&  entry)
      { return entry.arch; }
   QString  getSize             (const Package::Entry&  entry)
      { return entry.sizeText; }
   QString  getSummary          (const Package::Entry&  entry)
      { return entry.summary; }
   QString  getDescription      (const Package::Entry&  entry);
   QString  getGroup            (const Package::Entry&  entry);
   QString  getVendor           (const Package::Entry&  entry);
   QString  getBuildTime        (const Package::Entry&  entry)
      { return entry.buildTimeText; }
   QString  getBuildHost        (const Package::Entry&  entry);
   QString  getInstallTime      (const Package::Entry&  entry);
   QString  getBinaryRpm        (const Package::Entry&  entry);
   QString  getSourceRpm        (const Package::Entry&  entry);
   QString  getLicense          (const Package::Entry&  entry);
   QString  getPackager         (const Package::Entry&  entry);
   QString  getUrl              (const Package::Entry&  entry);
   QString  getRelocations      (const Package::Entry&  entry);
   QString  getProvides         (const Package::Entry&  entry);
   QString  getRequires         (const Package::Entry&  entry);
   QString  getConflicts        (const Package::Entry&  entry);
   QString  getObsoletes        (const Package::Entry&  entry);
   QString  getFiles            (const Package::Entry&  entry);
   QString  getChangeLog        (const Package::Entry&  entry);
   // For groups
   QString  getRequiredGroups   (const Package::Entry&  entry);
   QString  getDefaultMetaPkgs  (const Package::Entry&  entry);
   QString  getOptionalMetaPkgs (const Package::Entry&  entry);
   QString  getMandatoryPkgs    (const Package::Entry&  entry);
   QString  getDefaultPkgs      (const Package::Entry&  entry);
   QString  getOptionalPkgs     (const Package::Entry&  entry);
   QString  getConditionalPkgs  (const Package::Entry&  entry);
public:
   bool     hasPickledCache ();
   QString  getYumVar       (QString  name);
public:
   class EntryKey;
protected:
   typedef QHash<EntryKey,Package::Entry*>  EntryHash;
   Package::EntryList  list;
   EntryHash           hash;
protected:
   QStringList    collectRepos      (PyObject*              dict);
   void           useRepos          (PyObject*              repos,
                                     const YumReposUsed&    reposUsed);
   bool           collectField      (EntryHash&             hash,
                                     PyObject*              info,
                                     const char*            field,
                                     Package::State         state);
   bool           collectField      (EntryHash&             hash,
                                     PyObject*              info,
                                     char*                  field,
                                     Package::State         state);
   bool           collectSet        (EntryHash&             hash,
                                     PyObject*              set,
                                     Package::State         state);
   bool           collectPkg        (Package::Entry*&       entry,
                                     Package::State         state,
                                     PyObject*              pkg);
   bool           collectGroups     (EntryHash&             hash,
                                     PyObject*              groupInfo);
   bool           collectGroup      (Package::Entry*&       entry,
                                     PyObject*              groupObj,
                                     QString                name);
   bool           upToDateGroup     (const Package::Entry&  entry);
   bool           upToDateGroupGrps (const Package::Entry&  entry,
                                     const char*            attr_22);
   bool           upToDateGroupGrps (const Package::Entry&  entry,
                                     char*                  attr_22);
   bool           upToDateGroupPkgs (const Package::Entry&  entry,
                                     const char*            attr_22,
                                     const char*            attr_25);
   bool           upToDateGroupPkgs (const Package::Entry&  entry,
                                     char*                  attr_22,
                                     char*                  attr_25);
   QString        getPrco           (PyObject*              pkg,
                                     const char*            hdrKey,
                                     const char*            prcoKey);
   QString        getPrco           (PyObject*              pkg,
                                     const char*            hdrKey,
                                     char*                  prcoKey);
#if 0
   QString        getRpm            (PyObject*              pkg,
                                     char*                  rpmKey);
#endif
   QString        getRpmTime        (PyObject*              pkg,
                                     const char*            rpmKey);
   QString        getRpmList        (PyObject*              pkg,
                                     const char*            rpmKey);
   QString        getGroupGrps      (const Package::Entry&  entry,
                                     const char*            attr_22);
   QString        getGroupGrps      (const Package::Entry&  entry,
                                     char*                  attr_22);
   QString        getGroupPkgs      (const Package::Entry&  entry,
                                     const char*            attr_22,
                                     const char*            attr_25);
   QString        getGroupPkgs      (const Package::Entry&  entry,
                                     char*                  attr_22,
                                     char*                  attr_25);
   QString        makeSorted        (QStringList&           list);
   QString        makeSortedUniq    (QStringList&           list);
   PyObject*      getRpmHdrValue    (PyObject*              pkg,
                                     const char*            hdrKey);
   QString        getLocalPkg       (PyObject*              pkg);
   QString        getLocalHdr       (PyObject*              pkg);
   Package::Kind  getPkgKind        (PyObject*              pkg);
   PyObject*      getGroupItem_22   (const Package::Entry&  entry,
                                     char*                  attr);
   PyObject*      getGroupItem_25   (const Package::Entry&  entry,
                                     char*                  attr);
   bool           applyGroups       ();
   bool           applyGrpInstall   (const Package::Entry*  grp);
   bool           applyGrpUpdate    (const Package::Entry*  grp);
   bool           applyGrpRemove    (const Package::Entry*  grp);
   bool           applyGrpConflict  (const Package::Entry*  grp,
                                     Package::Entry*        pkg);
   void           applyGrpLog       (const Package::Entry*  grp,
                                     const Package::Entry*  pkg);
   void           applyGrpLogWas    (const Package::Entry*  grp,
                                     const Package::Entry*  pkg);
   void           applyGrpLogNot    (const Package::Entry*  grp,
                                     const Package::Entry*  pkg,
                                     Package::Action        action,
                                     QString                reason);
   PyObject*      requiredPkgs      (const Package::Entry*  grp);
   PyObject*      pkgTree           (const Package::Entry*  grp);
   PyObject*      allPkgs           (const Package::Entry*  grp);
   bool           makeLists         (PyObject*              yumIO,
                                     PyObject*&             tsInfo,
                                     PyObject*&             lists,
                                     PyObject*&             updated,
                                     PyObject*&             installed,
                                     PyObject*&             removed,
                                     PyObject*&             obsoleted,
                                     PyObject*&             depup,
                                     PyObject*&             depinst,
                                     PyObject*&             deprem);
   QString        simpleString      (PyObject*              pkg,
                                     const char*            key);
   QString        simpleString      (PyObject*              pkg,
                                     char*                  key);
   Package::Size  sizeOf            (PyObject*              pkg);
   QString        bytesFormat       (Package::Size          bytes);
   time_t         buildTimeOf       (PyObject*              pkg);
   char*          dateFormat        (time_t                 secs);
   char*          timeFormat        (time_t                 secs);
   QString        yumVersion        ();
   bool           hasDistribGroups  ();
   QString        pyString          (PyObject*              obj);
   QString        pyStringDec       (PyObject*              obj);
   QString        pyStringX         (PyObject*              obj);
   QString        pyStringXDec      (PyObject*              obj);
   QString        pyMappingString   (PyObject*              obj,
                                     const char*            attr);
   time_t         pyTime            (PyObject*              obj);
   time_t         pyTimeDec         (PyObject*              obj);
   PyObject*      pyDictGetItem     (PyObject*              dict,
                                     const QString&         key);
   void           log               (Log::Kind              kind,
                                     unsigned int           value,
                                     QString                text);
   virtual void   pyExceptExpect    (PyObject*              xcpt);
   virtual void   pyExceptClear     ();
   void           pythonException   (QString                text);
   void           pythonExceptionV  (QString                format);
protected:
   void  execWorkThread       (QEventLoop&          eventLoop,
                               QThread&             thread);
   bool  collectReal          (const YumReposUsed&  reposUsed);
   bool  applyReal            ();
   bool  collectFileListsReal ();
   bool  collectOtherDataReal ();
   friend class CollectThread;
   friend class ApplyThread;
   friend class CollectFileListsThread;
   friend class CollectOtherDataThread;
public:
   // These are public because they're callbacks from YumIOC.
   PyObject*  stdoutWrite           (PyObject*  self, PyObject*  args);
   PyObject*  stderrWrite           (PyObject*  self, PyObject*  args);
   PyObject*  log                   (PyObject*  self, PyObject*  args);
   PyObject*  errorlog              (PyObject*  self, PyObject*  args);
   PyObject*  collectProgress       (PyObject*  self, PyObject*  args);
   PyObject*  grabMeterStart        (PyObject*  self, PyObject*  args);
   PyObject*  grabMeterUpdate       (PyObject*  self, PyObject*  args);
   PyObject*  grabMeterEnd          (PyObject*  self, PyObject*  args);
   PyObject*  dsTsCheck             (PyObject*  self, PyObject*  args);
   PyObject*  dsRestartLoop         (PyObject*  self, PyObject*  args);
   PyObject*  dsEnd                 (PyObject*  self, PyObject*  args);
   PyObject*  userconfirm           (PyObject*  self, PyObject*  args);
   PyObject*  reportDownloadSize    (PyObject*  self, PyObject*  args);
   PyObject*  listTransaction       (PyObject*  self, PyObject*  args);
   PyObject*  tsInitCallback        (PyObject*  self, PyObject*  args);
   PyObject*  tsCallback            (PyObject*  self, PyObject*  args);
   PyObject*  tsEvent               (PyObject*  self, PyObject*  args);
   PyObject*  postTransactionOutput (PyObject*  self, PyObject*  args);
public:
   void       writeStdout                (QString          content);
   void       writeStderr                (QString          content);
protected:
   void       dsTsInfo                   (char*            msg);
   void       dsTsNewPkg                 (QString          name,
                                          unsigned int     numTs,
                                          bool             needHdr);
   PyObject*  write                      (PyObject*        self,
                                          PyObject*        args,
                                          Log::Kind        kind,
                                          unsigned int     value,
                                          QString&         buffer);
   void       write                      (QString          content,
                                          Log::Kind        kind,
                                          unsigned int     value,
                                          QString&         buffer);
   void       exceptionExpected          ();
protected:
   QString    pkgtupFormat               (PyObject*        pkgtup);
   QString    pkgtupRepoFormat           (PyObject*        pkgtup,
                                          QString          repoid);
   int        displayTransactionPkgs     (QString          action,
                                          PyObject*        txmbrs);
   int        displayTransactionPkgsRepo (QString          action,
                                          PyObject*        txmbrs);
   PyObject*  findRelation               (PyObject*        relatedto,
                                          const char*      relation);
   int        displayTransactionPkgsObs  (QString          action,
                                          PyObject*        txmbrs);
   void       noteTransactionPkgs        (Package::Action  action,
                                          PyObject*        txmbrs);
signals:
   void       collectedExtraData();
protected:
   void error   (const char*  text);
   void warning (const char*  text);
public:
   void error   (QString  text);
   void warning (QString  text);
public:
   Log::EntryList   logEntries;
   int              logStartOp;
   bool             collectedFileLists;
   bool             collectedOtherData;
protected:
   QWidget*         parent;
   YumIODialogs*    dialogs;
   YumIOCapturers*  capturers;
   PyObject*        rpm_dict;
   PyObject*        classYum;
   PyObject*        yumIO;
   PyObject*        reposDict;
   bool             locked;
   bool             yumVerKnown;
   QString          yumVer;
   struct {
      QString       stdoutBuffer;
      QString       stderrBuffer;
      Log::Kind     stderrKind;
      unsigned int  stderrValue;
   } wr;
   HProgressBar*    totalProgressBar;
   HProgressBar*    itemProgressBar;
   Progress*        pg;
   // Progress for reading repositories
   struct {
      QStringList  repos;
      QString      kind;
      int          act;   // Total number of acts is 2 * number of repos:
                          //    1 grab of metadata (maybe)
                          //    1 read of metadata
                          // If the download is skipped, it still is counted.
      int          total;
      QString      name;
   } repoProgress;
   // State for transaction
   struct {
      QString        descr;
      bool           confirmed;
      unsigned int   act;  
      QSet<QString>  updates;   // includes those from depresolve
      // State for checking dependencies (including hdr grabs)
      struct {
         unsigned int  totalTs;           // number of installs & erases
         unsigned int  totalGrabs;        // number of headers to download
         unsigned int  depSolveActs;      // depsolve acts for this iteration
         unsigned int  totalDepSolveActs; // depsolve acts for whole check
      } dep;
      // State for running transaction (including rpm grabs)
      struct {
         Package::Size  nominalGrabSize; // size of rpm's needed
         Package::Size  totalGrabSize;   // size of rpm's not in cache
         unsigned int   pgRGrabRpms;     // total progress for rpm grabs
         Package::Size  totalTsSize;     // size of installs & erases
         unsigned long  lastTs;          // last install/erase
      } run;
   } ts;
   class Blocker : QMutex {
   public:
      Blocker() : QMutex() {}
      void prepare() { lock(); }           /* called by creator thread */
      void release() { unlock(); }         /* called by creator thread */
      void block()   { lock(); unlock(); } /* called by created thread */
   };
   Blocker blocker;
};

#endif
