#include "YumIO.h"
#include "YumIOCapturers.h"
#include "globals.h"
#include <unistd.h>
#include <fcntl.h>

#define PIPE_READ   0
#define PIPE_WRITE  1

YumIOCapturers::YumIOCapturers(YumIO*  yumIO)
   : yumIO(yumIO)
{
   originalStdoutFd = dup(STDOUT_FILENO);
   originalStderrFd = dup(STDERR_FILENO);

   int  status = pipe(stdoutPipeFds);
   if (status == -1) {
      yumIO->error(tr("Unable to open pipe to capture rpm transaction output"));
      return;
   }
   status = pipe(stderrPipeFds);
   if (status == -1) {
      yumIO->error(tr("Unable to open pipe to capture rpm transaction output"));
      return;
   }

   status = dup2(stdoutPipeFds[PIPE_WRITE], STDOUT_FILENO);
   if (status == -1) {
      yumIO->error(tr("Unable to dup2 pipe to capture rpm transaction output"));
      return;
   }
   status = dup2(stderrPipeFds[PIPE_WRITE], STDERR_FILENO);
   if (status == -1) {
      yumIO->error(tr("Unable to dup2 pipe to capture rpm transaction output"));
      return;
   }

   fcntl(stdoutPipeFds[PIPE_READ], F_SETFL, O_NONBLOCK);
   fcntl(stderrPipeFds[PIPE_READ], F_SETFL, O_NONBLOCK);

   minion = new YumIOCapturersMinion(yumIO,
                                     stdoutPipeFds[PIPE_READ],
                                     stderrPipeFds[PIPE_READ]);

   minion->moveToThread(application->thread());

   minion->activate();
}

YumIOCapturers::~YumIOCapturers()
{
   int  status = dup2(originalStdoutFd, STDOUT_FILENO);
   if (status == -1) {
      yumIO->error(tr("Unable to dup2 to restore stderr"));
      return;
   }
   status = dup2(originalStderrFd, STDERR_FILENO);
   if (status == -1) {
      yumIO->error(tr("Unable to dup2 to restore stderr"));
      return;
   }
   
   close(stdoutPipeFds[PIPE_WRITE]);
   close(stderrPipeFds[PIPE_WRITE]);

   minion->deactivate();

   minion->deleteLater();
}
