#include "YumIODialogs.h"
#include "globals.h"
#include <QtCore/QThread>

YumIODialogs::YumIODialogs(QWidget*  parent)
{
   minion = new YumIODialogsMinion(parent);

   minion->moveToThread(application->thread());

   connect(minion, SIGNAL(setResult(int)),
           this,   SLOT(setResult(int)));

   connect(this,   SIGNAL(confirmSig(const QString&,
                                     const QString&)),
           minion, SLOT(confirm(const QString&,
                                const QString&)));
   connect(this,   SIGNAL(warningSig(const QString&,
                                     const QString&,
                                     const QString&,
                                     const QString&,
                                     const QString&,
                                     int,
                                     int)),
           minion, SLOT(warning(const QString&,
                                const QString&,
                                const QString&,
                                const QString&,
                                const QString&,
                                int,
                                int)));
   connect(this,   SIGNAL(criticalSig(const QString&,
                                      const QString&,
                                      const QString&,
                                      const QString&,
                                      const QString&,
                                      int,
                                      int)),
           minion, SLOT(critical(const QString&,
                                 const QString&,
                                 const QString&,
                                 const QString&,
                                 const QString&,
                                 int,
                                 int)));
   connect(this,   SIGNAL(scrollableSig(const QString&,
                                        const QString&)),
           minion, SLOT(scrollable(const QString&,
                                   const QString&)));
}

YumIODialogs::~YumIODialogs()
{
   minion->deleteLater();
}

void  YumIODialogs::setResult (int  value)
{
   gotResult = true;
   result = value;
   eventLoop.quit();
}

int  YumIODialogs::confirm (const QString&  caption,
                            const QString&  text)
{
   gotResult = false;
   emit confirmSig(caption, text);
   waitForResult();
   return result;
}

int  YumIODialogs::warning (const QString&  caption,
                            const QString&  text,
                            const QString&  button0Text,
                            const QString&  button1Text,
                            const QString&  button2Text,
                            int             defaultButtonNumber,
                            int             escapeButtonNumber)
{
   gotResult = false;
   emit warningSig(caption, text,
                   button0Text, button1Text, button2Text,
                   defaultButtonNumber, escapeButtonNumber);
   waitForResult();
   return result;
}

int  YumIODialogs::critical (const QString&  caption,
                             const QString&  text,
                             const QString&  button0Text,
                             const QString&  button1Text,
                             const QString&  button2Text,
                             int             defaultButtonNumber,
                             int             escapeButtonNumber)
{
   gotResult = false;
   emit criticalSig(caption, text,
                    button0Text, button1Text, button2Text,
                    defaultButtonNumber, escapeButtonNumber);
   waitForResult();
   return result;
}

void  YumIODialogs::scrollable (const QString&  caption,
                                const QString&  text)
{
   gotResult = false;
   emit scrollableSig(caption, text);
   waitForResult();
}

void  YumIODialogs::waitForResult ()
{
   if (!gotResult) {
      eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
   }
}

