#ifndef YumIODialogs_h
#define YumIODialogs_h

#include <QtCore/QObject>
#include <QtCore/QEventLoop>
#include <QtGui/QWidget>
#include "YumIODialogsMinion.h"

class YumIODialogs : public QObject {
   Q_OBJECT
public:
   explicit YumIODialogs(QWidget*  parent);
   ~YumIODialogs();
public:
   int   confirm    (const QString&  caption,
                     const QString&  text);
   int   warning    (const QString&  caption,
                     const QString&  text,
                     const QString&  button0Text         = QString(),
                     const QString&  button1Text         = QString(),
                     const QString&  button2Text         = QString(),
                     int             defaultButtonNumber = 0,
                     int             escapeButtonNumber  = -1);
   int   critical   (const QString&  caption,
                     const QString&  text,
                     const QString&  button0Text         = QString(),
                     const QString&  button1Text         = QString(),
                     const QString&  button2Text         = QString(),
                     int             defaultButtonNumber = 0,
                     int             escapeButtonNumber  = -1);
   void  scrollable (const QString&  caption,
                     const QString&  text);
public slots:
   void  setResult (int  value);
signals:
   int   confirmSig    (const QString&  caption,
                        const QString&  text);
   int   warningSig    (const QString&  caption,
                        const QString&  text,
                        const QString&  button0Text         = QString(),
                        const QString&  button1Text         = QString(),
                        const QString&  button2Text         = QString(),
                        int             defaultButtonNumber = 0,
                        int             escapeButtonNumber  = -1);
   int   criticalSig   (const QString&  caption,
                        const QString&  text,
                        const QString&  button0Text         = QString(),
                        const QString&  button1Text         = QString(),
                        const QString&  button2Text         = QString(),
                        int             defaultButtonNumber = 0,
                        int             escapeButtonNumber  = -1);
   void  scrollableSig (const QString&  caption,
                        const QString&  text);
protected:
   void  waitForResult  ();
protected:
   YumIODialogsMinion*  minion;
   QEventLoop           eventLoop;
   bool                 gotResult;
   int                  result;
};

// This class is just a handy way to declare a single object that, when
// destroyed (e.g. on exit from a routine) will reset the YumIODialogs.
class YumIODialogsUser {
public:
   explicit YumIODialogsUser(YumIODialogs*&     yumIODialogs,
                             QWidget*           parent) 
      : yumIODialogs(yumIODialogs)
   {
      oldYumIODialogs = yumIODialogs;
      yumIODialogs = new YumIODialogs(parent);
   }
   ~YumIODialogsUser() 
   { 
      delete yumIODialogs;
      yumIODialogs = oldYumIODialogs;
   }
protected:
   YumIODialogs*&  yumIODialogs;
   YumIODialogs*   oldYumIODialogs;
};

#endif
