#include "MainWindow.h"
#include <QtGui/QApplication>
#include "DefaultStyle.h"
#include "Share.h"
#include <QtGui/QIcon>
#include "globals.h"
#include <getopt.h>

static void
usage(void)
{
   printf("usage: %s [options...]\n", program);
   printf("\n");
   printf("NUU, Network Update Utility, is a graphical user interface which"
          " allows a user\n");
   printf("to make modifications to the installed rpm packages on a system.\n");
   printf("\n");
   printf("options:\n");
   printf("\n");
   printf("   -h, -H, --help           display this help information\n");
   printf("   -r,     --repo-editor    start by bringing up the repository"
                                       " configuration\n");
   printf("                            editor\n");
   printf("   -l,     --log            start with the log window displayed\n");
   printf("           --disablerepo=r  disable the specified repository, r\n");
   printf("           --enablerepo=r   enable the specified repository, r\n");
   printf("\n");
   exit(1);
}

int main(int    argc, 
         char*  argv[])
{
   DefaultStyle  defaultStyle(argc, argv);
   QApplication  app(argc, argv);
   defaultStyle.set(app, "plastique");

   app.setWindowIcon(QIcon(Share::absolute("pixmaps/nuu.icon.png")));

   application = &app;
   program     = basename(argv[0]);

   MainWindow*  mw = new MainWindow(NULL);

#define DISABLEREPO  256
#define ENABLEREPO   257

   {
      static const char           shortopts[] = "+hHrl";
      static const struct option  longopts[]  = {
         { "help",        no_argument,       NULL, 'h'         },
         { "repo-editor", no_argument,       NULL, 'r'         },
         { "log",         no_argument,       NULL, 'l'         },
         { "disablerepo", required_argument, NULL, DISABLEREPO },
         { "enablerepo",  required_argument, NULL, ENABLEREPO  },
         { NULL,          no_argument,       NULL, '\0'        }
      };

      opterr = 0;

      for (;;) {
         int  longindex;
         int  opt = getopt_long(argc, argv, shortopts, longopts, &longindex);

         if (opt == EOF) break;

         switch (opt) {
         case 'h':
         case 'H':
            usage();
            /*NOTREACHED*/
         case 'r':
            mw->optRepoEditor = true;
            break;
         case 'l':
            mw->optLog = true;
            break;
         case DISABLEREPO:
            mw->yumReposEnabled.append(YumRepoEnabled(false, optarg));
            break;
         case ENABLEREPO:
            mw->yumReposEnabled.append(YumRepoEnabled(true, optarg));
            break;
         case '?':
         default:
            if (optopt == '\0') {
               fprintf(stderr,
                       "%s: unrecogized option: %s\n", program, argv[optind-1]);
            } else {
               fprintf(stderr, 
                       "%s: unrecognized option letter: %c\n", program, optopt);
            }
            usage();
            /*NOTREACHED*/
         }
      }
   }

   if (optind < argc) {
      fprintf(stderr, "%s: unrecognized argument: %s\n", program, argv[optind]);
      usage();
      /*NOTREACHED*/
   }

   mw->show();

   return app.exec();
}
